/*
 * An interactive program to manage a (local or remote) tuple space.
 * You can use in, out, read (and print) from your console.
 * Type end when you're tired enough :)
 *
 * Usage: java InteractiveClient [-r [hostname [port]]]
 * 	-r: use a remote tuple server
 *  hostname: the host which runs the remote tuple server 
 *		(defaults to localhost)
 *	port: the port for the remote tuple server 
 *		(defaults to TupleNetConst.DEFAULT_PORT)
 *
 * (C) 1996 Davide Rossi
 */

import java.io.*;
import jada.tuple.*;
import jada.client.TupleClient;

public class InteractiveClient implements TupleNetConst {

	//program's entry point
	public static void main(String args[]) {

		//the tuple space
		TupleSpace tuple_space=null;
		//weather it is local or remote
		boolean remote=false;
		//where the remote tuple server is
		String tuplehost="localhost";
		//the remote port
		int port=DEFAULT_PORT;

		//input and ouput streams
		PrintStream cout=new PrintStream(System.out);
		DataInputStream cin=new DataInputStream(System.in);

		//check for -r switch
		for(int i=0; i<args.length; i++) {
			if(args[i].equals("-r")) {
				remote=true;
				if(i+1<args.length) {
					i++;
					tuplehost=args[i];
				}
				if(i+1<args.length) {
					i++;
					try {
						port=Integer.parseInt(args[i]);
					} catch(NumberFormatException e) {
						cout.println("wrong port format, default used");
					}
				}
			}
		}

		if(remote) {
			cout.println("Opening a remote tuple space to "+tuplehost);
			tuple_space=new TupleClient(tuplehost, port);
		} else {
			cout.println("Opening a local tuple space");
			tuple_space=new TupleSpace();
		}

		//the main loop
		boolean cont=true;
		while(cont) {
			cout.print("Tell me, master! "); cout.flush();
			try {
				String input=cin.readLine();
				if(input.startsWith("in_nb")) {
					Tuple match=new TupleString(input.substring(5));
					Tuple  tuple=tuple_space.in_nb(match);
					if(tuple==null) {
						cout.println("failed");
					} else {
						cout.println("in:"+tuple);
					}
				} else if(input.startsWith("in")) {
					Tuple match=new TupleString(input.substring(2));
					Tuple  tuple=tuple_space.in(match);
					cout.println("in:"+tuple);
				} else if(input.startsWith("out")) {
					Tuple  tuple=new TupleString(input.substring(3));
					tuple_space.out(tuple);
				} else if(input.startsWith("read_nb")) {
					Tuple match=new TupleString(input.substring(7));
					Tuple  tuple=tuple_space.read_nb(match);
					if(tuple==null) {
						cout.println("failed");
					} else {
						cout.println("read:"+tuple);
					}
				} else if(input.startsWith("read")) {
					Tuple match=new TupleString(input.substring(4));
					Tuple  tuple=tuple_space.read(match);
					cout.println("read:"+tuple);
				} else if(input.equals("print")) {
					tuple_space.print();
				} else if(input.equals("end")) {
					cont=false;
				}
			} catch(java.io.IOException e) {
				cont=false;
			}
		}
	}
}
